"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("../command");
const chalk_1 = require("chalk");
const fs = require("fs");
const chalk_2 = require("chalk");
const os = require("os");
const yaml_1 = require("../util/yaml");
const DIR = process.cwd();
// TODO: Add support for multiple stacks
class YalmTemplater extends command_1.Command {
    action(...args) {
        let command = args.pop();
        let action = args.shift();
        console.log(chalk_1.default.red("Working dir: ") + DIR);
        console.log(chalk_1.default.green("-- " + chalk_1.default.bold(action) + " --"));
        switch (action) {
            case "pack":
                this.pack(command);
                break;
        }
    }
    checkPathStart(path) {
        return (path.startsWith("/") || path.startsWith("."));
    }
    pack(obj) {
        return __awaiter(this, void 0, void 0, function* () {
            let template = obj.template || "template.yaml";
            let out = obj.outputFile || "template.build.yaml";
            let dir = obj.inputDir || "test";
            let absoluteTemplatePath = (!this.checkPathStart(template)) ? DIR + "/" + template : template;
            let absoluteBuildPath = (!this.checkPathStart(out)) ? DIR + "/" + out : out;
            let absoluteDirPath = (!this.checkPathStart(dir)) ? DIR + "/" + dir : dir;
            let dirs = fs.readdirSync(absoluteDirPath);
            if (!fs.existsSync(absoluteTemplatePath)) {
                console.log(chalk_1.default.red("No template file!, " + absoluteTemplatePath));
                return;
            }
            let templateContent = fs.readFileSync(absoluteTemplatePath, "UTF-8");
            let snips = "";
            for (let dir of dirs) {
                let checking = absoluteDirPath + "/" + dir;
                let file = "/templ-snip.yaml";
                let absolute = checking + file;
                if (!fs.existsSync(absolute)) {
                    console.log(chalk_1.default.magenta("Checked: ") + dir + chalk_1.default.red(" \u{274C}"));
                    continue;
                }
                console.log(chalk_1.default.magenta("Checked: ") + dir + chalk_1.default.greenBright(" \u{2714}"));
                let content = fs.readFileSync(absolute, "UTF-8");
                if (!(yield yaml_1.YamlUtil.isValidYaml(content))) {
                    console.log(chalk_1.default.magenta("Valid YAML: ") + dir + chalk_1.default.red(" \u{274C}"));
                }
                else {
                    console.log(chalk_1.default.magenta("Valid YAML: ") + dir + chalk_1.default.greenBright(" \u{2714}"));
                }
                snips += os.EOL + content;
                console.log(chalk_1.default.magenta("Loading: ") + dir + chalk_2.default.greenBright(" done"));
            }
            let newTemplate = templateContent.replace("resource: '<replace>'", snips);
            if (!(yield yaml_1.YamlUtil.isValidYaml(newTemplate))) {
                console.log(chalk_1.default.red("Not a valid YAML file!"));
            }
            else {
                console.log(chalk_1.default.green("Valid YAML file!"));
            }
            console.log(chalk_1.default.green("-- " + chalk_1.default.bold("Done: ")) + "File written to: " + absoluteBuildPath + chalk_2.default.green(" --"));
            fs.writeFileSync(absoluteBuildPath, newTemplate);
        });
    }
    setConfiguration() {
        this.command = "yaml-packer <action>";
        this.options = ["-t, --template <value>",
            "-o, --output-file <value>",
            "-i, --input-dir <value>",
            "-c --validate"];
        this.description = "Fetches templ-snips.yaml files from the sub-directories of the given --input-dir and replaces resource: '<replace>' with the content from snips in the template file";
    }
}
exports.default = YalmTemplater;
